# ruff: noqa: I001
from .collection_util import CollectionUtil as CollectionUtil
from .debug_util import DebugUtil as DebugUtil
from .func_util import FuncUtil as FuncUtil
from .log_util import LogUtil as LogUtil
from .mail_util import MailUtil as MailUtil
from .object_util import ObjectUtil as ObjectUtil
from .random_util import RandomUtil as RandomUtil
from .str_util import StrUtil as StrUtil
from .throttle import Throttle as Throttle
from .time_util import TimeUtil as TimeUtil

from .check_util import CheckUtil as CheckUtil
from .io_util import IOUtil as IOUtil
from .config_util import ConfigUtil as ConfigUtil



from enum import IntEnum


class BaseResultErrorType(IntEnum):
    RES_OK = 0

    RES_ERR_BAD_FORMAT = 1000
    RES_ERR_SERVER_ERROR = 1001
    RES_ERR_NO_VALID_TOKEN = 1002
    RES_ERR_NOT_AUTHORIZED = 1003
    RES_ERR_DB_OPERATION = 1004
    RES_ERR_NAME_IN_USE = 1005
    RES_ERR_INVALID_STATE = 1007
    RES_ERR_TIMEOUT = 1008
    RES_ERR_CONNECTION_END = 1009
    RES_ERR_INVALID = 1011
    RES_ERR_TOO_FREQUENT = 1012
    RES_ERR_CANCELLED = 1013

    RES_ERR_RESOURCE_NOT_EXIST = 1020
    RES_ERR_RESOURCE_NOT_AVAILABLE = 1021
    RES_ERR_RESOURCE_NOT_ENOUGH = 1022
    RES_ERR_INVALID_CONFIG = 1030

    # auth
    RES_ERR_WRONG_PASSWORD = 2000
    RES_ERR_WRONG_ACTIVATION_CODE = 2001

    # users
    RES_ERR_USER_NOT_EXIST = 3000
    RES_ERR_EMAIL_IN_USE = 3001
    RES_ERR_USER_IN_USE = 3002
    RES_ERR_USER_OFFLINE = 3003

    # frontend




import json
from typing import Any


class StrUtil:
    @staticmethod
    def pprint(obj: Any, not_process_str: bool = True) -> str:
        if not obj:
            return str(obj)
        if not_process_str and isinstance(obj, str):
            return obj
        temp = json.dumps(obj, indent=4, ensure_ascii=False, default=str)
        return temp



import os

from hpyhrtbase.config import Params, params_from_file


def test_params_from_file(configs_dir):
    config_path = os.path.join(configs_dir, "example_config_001.conf")
    params = params_from_file(config_path)

    assert params.key1 == "value1"
    assert params.key2 == "value2"
    assert params.key_dict.inner_key1 == "inner_value1"


def test_params_to_dict():
    inner_params = Params(inner_key1="inner_value1")
    params = Params(key1="value1", key2="value2")
    params["key_dict"] = inner_params

    assert params.key1 == "value1"
    assert params.key2 == "value2"
    assert params.key_dict.inner_key1 == "inner_value1"

    params_dict = params.as_dict()

    assert params_dict["key1"] == "value1"
    assert params_dict["key2"] == "value2"
    assert params_dict["key_dict"]["inner_key1"] == "inner_value1"



from hpyhrtbase import log
from hpyhrtbase.config import Params


def test_setup_logging(tmp_path):
    params = Params()
    params.project_dir = str(tmp_path)

    app_log_path = tmp_path / "logs" / "app.log"

    log.setup_logging(params)

    log_content = app_log_path.read_text(encoding="utf-8")
    assert "info: App start logging" in log_content



import codecs
import logging
import mimetypes
import smtplib
import time
from collections.abc import Iterable
from email import encoders
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from typing import Any


class MailUtil:
    def __init__(
        self,
        server: str,
        user: str,
        password: str,
        str_from: str | None = None,
        use_ssl: bool = True,
    ):
        self.server = server
        self.user = user
        self.password = password
        if str_from:
            self.str_from = str_from
        else:
            self.str_from = user
        self.use_ssl = use_ssl

    def send_mail(
        self,
        str_to: str,
        str_subject: str,
        plain_text: str,
        html_text: str | None = None,
        attachments: Iterable[Any] | None = None,
        try_num: int = 1,
    ) -> bool:
        outer = MIMEMultipart("alternative")
        outer["Subject"] = str_subject
        outer["From"] = self.str_from
        outer["To"] = str_to
        outer.preamble = "You will not see this in a MIME-aware mail reader.\n"

        part1 = MIMEText(plain_text, "plain")
        outer.attach(part1)

        if html_text:
            part2 = MIMEText(html_text, "html")
            outer.attach(part2)

        if not attachments:
            attachments = []

        for path in attachments:
            content_type, encoding = mimetypes.guess_type(path)

            if content_type is None or encoding is not None:
                # No guess could be made, or the file is encoded (compressed), so
                # use a generic bag-of-bits type.
                content_type = "application/octet-stream"

            maintype, subtype = content_type.split("/", 1)
            msg: Any
            if maintype == "text":
                with codecs.open(path, "r", "utf-8") as file_handle:
                    # Note: we should handle calculating the charset
                    msg = MIMEText(file_handle.read(), _subtype=subtype)
            elif maintype == "image":
                with open(path, "rb") as file_handle:
                    msg = MIMEImage(file_handle.read(), _subtype=subtype)
            elif maintype == "audio":
                with open(path, "rb") as file_handle:
                    msg = MIMEAudio(file_handle.read(), _subtype=subtype)
            else:
                with open(path, "rb") as file_handle:
                    msg = MIMEBase(maintype, subtype)
                    msg.set_payload(file_handle.read())
                # Encode the payload using Base64
                encoders.encode_base64(msg)
            # Set the filename parameter
            msg.add_header("Content-Disposition", "attachment", filename=path)
            outer.attach(msg)

        for i in range(try_num):
            try:
                if i != 0:
                    time.sleep(2)
                smtp: smtplib.SMTP
                if self.use_ssl:
                    smtp = smtplib.SMTP_SSL(self.server)
                else:
                    smtp = smtplib.SMTP()
                    smtp.connect(self.server)
                smtp.login(self.user, self.password)
                dst_list = str_to.split(",")
                if len(dst_list) <= 8:
                    smtp.sendmail(self.str_from, dst_list, outer.as_string())
                else:
                    smtp.sendmail(self.str_from, dst_list[0:8], outer.as_string())
                    smtp.sendmail(self.str_from, dst_list[8:], outer.as_string())
                smtp.quit()
                return True
            except Exception as e:
                logging.warning(str(e))
        return False



import os

from hpyhrtbase import hpyhrt_context
from hpyhrtbase.config import params_from_file


def test_global_context():
    global_context = hpyhrt_context.get_global_context()
    global_context.key1 = "value1"


def test_config_inst(configs_dir):
    config_path = os.path.join(configs_dir, "example_config_001.conf")
    params = params_from_file(config_path)

    hpyhrt_context.set_config_inst(params)

    config_inst = hpyhrt_context.get_config_inst()

    assert config_inst.key1 == "value1"



import os

from hpyhrtbase import hpyhrt_context, init_app_base


def test_init_app_base(configs_dir):
    config_path = os.path.join(configs_dir, "default_config.conf")
    init_app_base.init_app_base(config_path, check_dir_names=["configs"])

    config_inst = hpyhrt_context.get_config_inst()

    assert config_inst.key1 == "value1"



from __future__ import annotations

from enum import IntEnum

from pydantic import BaseModel


class ErrorInfo(BaseModel):
    code: int
    message: str
    detail: str | None = None

    @classmethod
    def from_enum(cls, err_enum: IntEnum, detail: str | None = None) -> ErrorInfo:
        return ErrorInfo(code=int(err_enum), message=err_enum.name, detail=detail)



import logging
import time
import traceback
from collections.abc import Callable
from datetime import datetime, timedelta, timezone
from functools import wraps
from logging import Logger
from typing import Any, ParamSpec, TypeVar, cast

from hpyhrtbase.model import ErrorInfoException, ErrorResponse

mod_logger = logging.getLogger(__name__)

P = ParamSpec("P")
T = TypeVar("T")


class FuncUtil:
    @staticmethod
    def deco_handle_error_info_exception(f: Callable[P, T]) -> Callable[P, T]:
        @wraps(f)
        def f_handle_error_info_exception(*args: P.args, **kwargs: P.kwargs) -> T:
            try:
                return f(*args, **kwargs)
            except ErrorInfoException as e:
                return cast(T, ErrorResponse(error=e.error_info))

        return f_handle_error_info_exception

    @staticmethod
    def retry(
        ExceptionToCheck: type[Exception],
        tries: int = 4,
        delay: float = 3,
        backoff: float = 2,
        logger: Logger | None = None,
    ) -> Callable[[Callable[P, T]], Callable[P, T]]:
        """Retry calling the decorated function using an exponential backoff.

        http://www.saltycrane.com/blog/2009/11/trying-out-retry-decorator-python/
        original from: http://wiki.python.org/moin/PythonDecoratorLibrary#Retry

        :param ExceptionToCheck: the exception to check. may be a tuple of
            exceptions to check
        :type ExceptionToCheck: Exception or tuple
        :param tries: number of times to try (not retry) before giving up
        :type tries: int
        :param delay: initial delay between retries in seconds
        :type delay: int
        :param backoff: backoff multiplier e.g. value of 2 will double the delay
            each retry
        :type backoff: int
        :param logger: logger to use. If None, print
        :type logger: logging.Logger instance
        """

        def deco_retry(f: Callable[P, T]) -> Callable[P, T]:
            @wraps(f)
            def f_retry(*args: P.args, **kwargs: P.kwargs) -> T:
                mtries, mdelay = tries, delay
                while mtries > 1:
                    try:
                        return f(*args, **kwargs)
                    except ExceptionToCheck as e:
                        mod_logger.info(traceback.format_exc())
                        msg = f"{str(e)}, Retrying in {mdelay:.2f} seconds..."
                        if logger:
                            logger.warning(msg)
                        else:
                            mod_logger.warning(msg)
                        time.sleep(mdelay)
                        mtries -= 1
                        mdelay *= backoff
                return f(*args, **kwargs)

            return f_retry  # true decorator

        return deco_retry

    @staticmethod
    def retry_until_timeout(func: Callable[..., Any], timeout: float, sleep_time: float = 1) -> Any:
        final_time = datetime.now(timezone.utc) + timedelta(seconds=timeout)

        first = True
        while datetime.now(timezone.utc) <= final_time:
            if first:
                rsp = func()
                first = False
            else:
                time.sleep(sleep_time)
                rsp = func()

            if rsp is None:
                continue
            else:
                return rsp

        return None

    @staticmethod
    def run_ignore_exception(func: Callable[..., Any]) -> None:
        try:
            func()
        except Exception:
            pass



from typing import Any


class ObjectUtil:
    @staticmethod
    def setattr_if_not_exist(obj: object, name: str, value: Any) -> None:
        if not hasattr(obj, name):
            setattr(obj, name, value)



from __future__ import annotations

from enum import IntEnum

from .error_info import ErrorInfo


class ErrorInfoException(Exception):
    def __init__(self, error_info):
        super().__init__()

        self.error_info = error_info

    @classmethod
    def from_enum(cls, err_enum: IntEnum, detail: str | None = None) -> ErrorInfoException:
        error_info = ErrorInfo(code=int(err_enum), message=err_enum.name, detail=detail)
        return ErrorInfoException(error_info)



from typing import Any, Literal


class CollectionUtil:
    @staticmethod
    def dict_extract_by_keys(in_dict: dict[str, Any], key_list: list[str]) -> dict[str, Any]:
        target_dict = {}

        for key in key_list:
            if key in in_dict:
                target_dict[key] = in_dict[key]

        return target_dict

    @staticmethod
    def dict_get_by_path(key_path: str, cur_ns: dict[str, Any]) -> tuple[Any, bool]:
        key_path_items = key_path.split(".")

        for i in range(len(key_path_items) - 1):
            if key_path_items[i] not in cur_ns or not isinstance(cur_ns[key_path_items[i]], dict):
                return None, False
            cur_ns = cur_ns[key_path_items[i]]

        return cur_ns.get(key_path_items[-1]), True

    @staticmethod
    def dict_ensure_key_path(key_path: str, cur_ns: dict[str, Any]) -> tuple[str, Any]:
        key_path_items = key_path.split(".")

        for i in range(len(key_path_items) - 1):
            if key_path_items[i] not in cur_ns or cur_ns[key_path_items[i]] is None:
                cur_ns[key_path_items[i]] = {}
            cur_ns = cur_ns[key_path_items[i]]

        return key_path_items[-1], cur_ns

    @staticmethod
    def dict_update_if_not_exist(target: dict[str, Any], source: dict[str, Any], keys: list[str]) -> None:
        for key in keys:
            if key not in target and key in source:
                target[key] = source[key]

    @staticmethod
    def dict_update_on_mode(target: dict[str, Any], other: dict[str, Any], mode: Literal["override", "merge"] = "override") -> None:
        if mode == "override":
            target.update(other)
        elif mode == "merge":
            for key, value in other.items():
                if key not in target:
                    target[key] = value
                elif not (isinstance(target[key], dict) and isinstance(value, dict)):
                    target[key] = value
                else:
                    CollectionUtil.dict_update_on_mode(target[key], value, mode)
        else:
            raise Exception(f"invalid mode {mode}")



from types import SimpleNamespace
from typing import Any

from hpyhrtbase.config import Params

config_inst: Params | None = None
global_context: Any = SimpleNamespace()


def reset() -> None:
    global config_inst, global_context
    config_inst = None
    global_context = SimpleNamespace()


def get_global_context() -> Any:
    return global_context


def set_config_inst(inst: Params) -> None:
    global config_inst
    config_inst = inst


def get_config_inst() -> Params:
    global config_inst

    if config_inst is None:
        raise ValueError("config not parsed yet?")

    return config_inst



import hashlib
import random
import uuid


class RandomUtil:
    @staticmethod
    def gen_guid() -> str:
        return str(uuid.uuid4())

    @staticmethod
    def hash_sha1(input_str: str, private: str | None = None) -> str:
        if not private:
            private = "Work hard, study hard, play hard, enjoy the life"
        m = hashlib.sha1()
        m.update(private.encode())
        m.update(input_str.encode())
        m.update(private.encode())
        digest = m.hexdigest()
        return digest

    @staticmethod
    def is_valid_uuid(uuid_to_test: str, version: int = 4) -> bool:
        try:
            uuid_obj = uuid.UUID(uuid_to_test, version=version)
        except ValueError:
            return False

        return str(uuid_obj) == uuid_to_test

    @staticmethod
    def random_num_seq(seq_len: int) -> str:
        result = ""
        for i in range(seq_len):
            result += "%d" % random.randint(0, 9)
        return result



import logging
import time
from datetime import datetime
from urllib.parse import urlparse

default_delay_maps: dict[str, float] = {
    'dfcfw.com': 1,
    'eastmoney.com': 1.2,
    'sina.com.cn': 3,
    'sohu.com': 2,
    'tushare.pro': 0.3,
    '163.com': 1,
}

mod_logger = logging.getLogger(__name__)


class Throttle:
    def __init__(self, delay_maps: dict[str, float] | None = None):
        if delay_maps is not None:
            self.delay_maps = delay_maps.copy()
        else:
            self.delay_maps = default_delay_maps.copy()
        self.domains_last_accessed: dict[str, datetime] = {}

    def update_delay_maps(self, delay_maps_update: dict[str, float]) -> None:
        self.delay_maps.update(delay_maps_update)

    def calc_wait(self, url: str) -> tuple[str, float]:
        parsed = urlparse(url)
        hostname = parsed.hostname

        assert hostname is not None

        parts = hostname.split('.')

        if len(parts) >= 3 and ".".join(parts[-2:]) in ["com.cn"]:
            effective_domain = ".".join(parts[-3:])
        else:
            effective_domain = ".".join(parts[-2:])

        if effective_domain not in self.delay_maps:
            return effective_domain, 0

        last_accessed = self.domains_last_accessed.get(effective_domain)
        delay = self.delay_maps[effective_domain]

        if last_accessed is not None:
            cur_time = datetime.now()
            elapsed = (cur_time - last_accessed).total_seconds()
            sleep_secs = delay - elapsed
        else:
            sleep_secs = 0

        return effective_domain, sleep_secs

    def wait(self, url: str) -> None:
        domain, sleep_secs = self.calc_wait(url)

        if sleep_secs > 0:
            mod_logger.info(f"wait: domain {domain}, sleep {sleep_secs}")
            time.sleep(sleep_secs)
        self.domains_last_accessed[domain] = datetime.now()



class DebugUtil:
    @staticmethod
    def reset_app() -> None:
        from hpyhrtbase import init_app_base

        init_app_base._init_app_base_done = False
        init_app_base._logging_setup_done = False



import time
from collections.abc import Generator
from contextlib import contextmanager
from datetime import date, datetime
from types import SimpleNamespace


class TimeUtil:
    @staticmethod
    def convert_date_int_to_date_obj(date_int: int) -> date:
        date_obj = date(date_int//10000, (date_int//100)%100, date_int%100)
        return date_obj

    @staticmethod
    def convert_date_obj_to_date_int(date_obj: date) -> int:
        date_int = date_obj.year * 10000 + date_obj.month * 100 + date_obj.day
        return date_int

    @staticmethod
    def datetime_fromisoformat(datetime_str: datetime | str) -> datetime | None:
        try:
            if isinstance(datetime_str, datetime):
                return datetime_str

            if datetime_str[-1] == "Z":
                datetime_str = datetime_str[:-1] + "+00:00"

            parsed_datetime = datetime.fromisoformat(datetime_str)
        except Exception:
            return None
        return parsed_datetime

    @staticmethod
    def get_current_time_isoformat() -> str:
        local_datetime = datetime.now()
        return local_datetime.isoformat()

    @staticmethod
    def get_current_time_ms() -> int:
        millis = int(round(time.time() * 1000))
        return millis

    @staticmethod
    def get_current_time_monotonic() -> float:
        return time.monotonic()

    @contextmanager
    @staticmethod
    def timeit() -> Generator[SimpleNamespace]:
        start_time = time.monotonic()
        ns = SimpleNamespace()
        yield ns
        duration = time.monotonic() - start_time
        ns.duration = duration


if __name__ == "__main__":
    print(TimeUtil.get_current_time_isoformat())



from __future__ import annotations

import json
import logging
import os
import types
from collections.abc import Iterable
from typing import Any, cast

import pyhocon

logging.basicConfig(format='%(asctime)s  %(levelname)-8s [%(name)s] %(module)s:%(funcName)s:%(lineno)d: %(message)s',
                level=logging.INFO)


class Params:
    """Params handler object.

    This functions as a nested dict, but allows for seamless dot-style access, similar to tf.HParams but without the type validation. For example:

    p = Params(name="model", num_layers=4)
    p.name  # "model"
    p['data'] = Params(path="file.txt")
    p.data.path  # "file.txt"
    """

    @staticmethod
    def clone(source: Any, strict: bool = True) -> Params | None:
        if isinstance(source, pyhocon.ConfigTree):
            return Params(**source.as_plain_ordered_dict())
        elif isinstance(source, Params):
            return Params(**source.as_dict())
        elif isinstance(source, dict):
            return Params(**source)
        elif strict:
            raise ValueError("Cannot clone from type: " + str(type(source)))
        else:
            return None

    def __getattr__(self, name: str) -> Any:
        raise AttributeError(f"{name} not found")

    def __setattr__(self, name: str, value: Any) -> None:
        super().__setattr__(name, value)

    def __getitem__(self, k: str) -> Any:
        return getattr(self, k)

    def __contains__(self, k: str) -> bool:
        return k in self._known_keys

    def __setitem__(self, k: str, v: Any) -> None:
        assert isinstance(k, str)
        if isinstance(self.get(k, None), types.FunctionType):
            raise ValueError(f"Invalid parameter name (overrides reserved name '{k}').")

        converted_val = Params.clone(v, strict=False)

        if converted_val is not None:
            setattr(self, k, converted_val)
        else:  # plain old data
            setattr(self, k, v)

        self._known_keys.add(k)

    def __delitem__(self, k: str) -> None:
        if k not in self:
            raise ValueError(f"Parameter {k} not found.")
        delattr(self, k)
        self._known_keys.remove(k)

    def __init__(self, **kw: Any) -> None:
        """Create from a list of key-value pairs."""
        self._known_keys: set[str] = set()
        for k, v in kw.items():
            self[k] = v

    def get(self, k: str, default: Any = None) -> Any:
        return getattr(self, k, default)

    def keys(self) -> list[str]:
        return sorted(self._known_keys)

    def as_dict(self) -> dict[str, Any]:
        """Recursively convert to a plain dict."""
        def convert(v: Any) -> Any:
            return v.as_dict() if isinstance(v, Params) else v
        return {k: convert(self[k]) for k in self.keys()}

    def __repr__(self) -> str:
        return self.as_dict().__repr__()

    def __str__(self) -> str:
        return json.dumps(self.as_dict(), indent=2, sort_keys=True)


# Argument handling is as follows:
# 1) read config file into pyhocon.ConfigTree
# 2) merge overrides into the ConfigTree
# 3) validate specific parameters with custom logic


def params_from_file(config_files: str | Iterable[str],
                     overrides: str | None = None) -> Params:
    config_string = ''

    if isinstance(config_files, str):
        config_files = [config_files]

    for config_file in config_files:
        with open(config_file, encoding='utf-8') as fd:
            print("")
            logging.info(f"Loading config from {config_file}")
            config_string += fd.read()
            config_string += '\n'

    if overrides:
        logging.info(f"Config overrides: {overrides}")
        # Append overrides to file to allow for references and injection.
        config_string += "\n"
        config_string += overrides

    basedir = os.path.dirname(config_file)  # directory context for includes
    config = pyhocon.ConfigFactory.parse_string(config_string, basedir=basedir)
    return cast(Params, Params.clone(config))



from .common_result import CommonResult as CommonResult
from .error_info import ErrorInfo as ErrorInfo
from .error_info_exception import ErrorInfoException as ErrorInfoException
from .error_response import ErrorResponse as ErrorResponse
from .errors import BaseResultErrorType as BaseResultErrorType



import logging
import logging.config
import os
from collections.abc import Callable
from typing import Any

from hpyhrtbase import config, hpyhrt_context, log
from hpyhrtbase.utils import IOUtil, Throttle

__all__ = ["init_app_base"]

_init_app_base_done = False


def reset() -> None:
    global _init_app_base_done
    _init_app_base_done = False


def init_app_base(
    config_path: str,
    init_app_default_configs: Callable[[], Any] | None = None,
    check_dir_names: list[str] | None = None,
) -> None:
    """解析config, 配置app特有的缺省配置, 创建缺省目录, 配置log"""
    global _init_app_base_done
    log_prefix = "init_app_base:"

    if _init_app_base_done:
        raise Exception(f"invalid state, _init_app_base_done {_init_app_base_done}")

    _init_app_base_done = True

    pid = os.getpid()

    log_msg = f"{log_prefix} pid {pid}, Enter with config_path {config_path}"
    print(log_msg)
    logging.info(log_msg)

    config_file = IOUtil.find_abs_path(config_path)

    if not config_file:
        raise Exception("Can't find config file")

    root_dir = IOUtil.find_root_dir(config_file, check_dir_names=check_dir_names)

    overrides = f"root_dir={root_dir}"
    overrides = overrides.replace("\\", "\\\\")
    config_inst = config.params_from_file(config_file, overrides)

    hpyhrt_context.set_config_inst(config_inst)

    if init_app_default_configs:
        init_app_default_configs()

    init_base_default_configs(config_inst)

    global_context = hpyhrt_context.get_global_context()
    global_context.throttle = Throttle()

    # Logistics #
    if hasattr(config_inst, "project_dir"):
        IOUtil.maybe_make_dir(config_inst.project_dir)
    if hasattr(config_inst, "data_cache_dir"):
        IOUtil.maybe_make_dir(config_inst.data_cache_dir)
    if hasattr(config_inst, "output_dir"):
        IOUtil.maybe_make_dir(config_inst.output_dir)

    log.setup_logging(config_inst)

    logging.info(f"{log_prefix} Done")


def init_base_default_configs(config_inst: config.Params) -> None:
    if not hasattr(config_inst, "data_dir"):
        config_inst.data_dir = os.path.join(config_inst.root_dir, "data")

    if not hasattr(config_inst, "norm_timeout"):
        config_inst.norm_timeout = 10

    if not hasattr(config_inst, "data_cache_dir"):
        config_inst.data_cache_dir = os.path.join(config_inst.project_dir, "data_cache")

    if not hasattr(config_inst, "output_dir"):
        config_inst.output_dir = os.path.join(config_inst.project_dir, "outputs")



import json
import math
from typing import Any, NoReturn

from hpyhrtbase import hpyhrt_context
from hpyhrtbase.utils import StrUtil


class CheckUtil:
    @staticmethod
    def assert_for_log(condition: Any, error_message: str) -> None:
        assert condition, error_message

    @staticmethod
    def check_dict_expected(
        expected: dict[str, Any], other: dict[str, Any], ignore_order: bool = False, order_by_key: Any = None
    ) -> bool:
        other_value = None
        list_idx = -1

        for key, local_value in expected.items():
            keys = key.split(".")
            for idx, sub_key in enumerate(keys):
                list_idx_start = sub_key.find("[")

                if list_idx_start >= 0:
                    list_idx_end = sub_key.find("]")
                    list_idx = int(sub_key[list_idx_start + 1 : list_idx_end])
                    sub_key = sub_key[:list_idx_start]

                if idx == 0:
                    other_value = other.get(sub_key, None)
                else:
                    if other_value is None:
                        hpyhrt_context.get_robot_logger().info(
                            f"other doesn't have valid value for key {key}, other \n{StrUtil.pprint(other)}")
                        return False

                    if type(other_value) is str:
                        other_value = json.loads(other_value)

                    other_value = other_value.get(sub_key, None)

                if list_idx_start >= 0:
                    if other_value is None:
                        hpyhrt_context.get_robot_logger().info(
                            f"other doesn't have valid value for key {key}, other \n{StrUtil.pprint(other)}")
                        return False

                    if not isinstance(other_value, list):
                        hpyhrt_context.get_robot_logger().info(
                            f"other key {key} is not list, other \n{StrUtil.pprint(other)}")
                        return False
                    if len(other_value) < list_idx:
                        hpyhrt_context.get_robot_logger().info(
                            f"other key {key} not contain idx {list_idx}, other \n{StrUtil.pprint(other)}")
                        return False
                    other_value = other_value[list_idx]

            if other_value is None and local_value is None:
                return True
            elif other_value is None or local_value is None:
                return False

            if not isinstance(local_value, dict):
                if isinstance(local_value, list) and ignore_order:
                    if isinstance(local_value[0], dict):
                        local_value = sorted(local_value, key=order_by_key)
                        other_value = sorted(other_value, key=order_by_key)
                    else:
                        local_value = sorted(local_value)
                        other_value = sorted(other_value)

                if local_value != other_value:
                    hpyhrt_context.get_robot_logger().info(
                        f"key {key} not match, expect {local_value}, got {other_value}, other \n{StrUtil.pprint(other)}")
                    return False
                continue

            for inner_key in local_value.keys():
                inner_local_value = local_value.get(inner_key)
                inner_other_value = other_value.get(inner_key, None)

                if (
                    isinstance(inner_local_value, list)
                    and isinstance(inner_other_value, list)
                    and ignore_order
                ):
                    if isinstance(inner_local_value[0], dict):
                        inner_local_value = sorted(inner_local_value, key=order_by_key)
                        inner_other_value = sorted(inner_other_value, key=order_by_key)
                    else:
                        inner_local_value = sorted(inner_local_value)
                        inner_other_value = sorted(inner_other_value)

                if inner_local_value != inner_other_value:
                    hpyhrt_context.get_robot_logger().info(
                        f"key {key}.{inner_key} not match, expect {inner_local_value}, got {inner_other_value}")
                    return False

        return True

    @staticmethod
    def check_json(input_str: str) -> bool:
        try:
            json.loads(input_str)
            return True
        except Exception:
            return False

    @staticmethod
    def equal(lhs: Any, rhs: Any) -> bool:
        if isinstance(lhs, float) and isinstance(rhs, float):
            return math.isclose(lhs, rhs, abs_tol=0.0001)
        elif lhs == rhs:
            return True
        return False

    @staticmethod
    def halt(description: str = "halt") -> NoReturn:
        hpyhrt_context.get_robot_logger().error(f"Enter halt: {description}")

        raise Exception(description)



import json
import logging
import os
import shutil
import time
import zipfile
from datetime import datetime, timedelta
from typing import Any

import requests

from hpyhrtbase import hpyhrt_context

mod_logger = logging.getLogger(__name__)


class IOUtil:
    @staticmethod
    def download_file(url: str, local_filepath: str, *, max_retry: int = 2) -> None:
        for i in range(max_retry):
            try:
                with requests.get(url, stream=True, verify=False) as r:
                    r.raise_for_status()
                    total_size_in_bytes = int(r.headers.get("content-length", 0))
                    total_written = 0
                    with open(local_filepath, "wb") as f:
                        for chunk in r.iter_content(chunk_size=8192):
                            if chunk:  # filter out keep-alive new chunks
                                f.write(chunk)
                                total_written += len(chunk)
                                mod_logger.debug(
                                    f"downloading {url}: written {total_written} / {total_size_in_bytes}"
                                )
                    mod_logger.debug(f"download_file {url} to {local_filepath} succeed")
                return
            except Exception as e:
                if i == (max_retry - 1):
                    mod_logger.warning(
                        f"download_file {url} to {local_filepath} failed: {e}"
                    )
                    raise e

    @staticmethod
    def download_if_needed(src_url: str, target_dir: str, *, max_retry: int = 2) -> str:
        base_name = src_url.rsplit("/", 1)[1]
        target_path = os.path.join(target_dir, base_name)

        if os.path.exists(target_path):
            return target_path

        target_path_temp = target_path + ".temp"

        IOUtil.download_file(src_url, target_path_temp, max_retry=max_retry)
        os.rename(target_path_temp, target_path)

        return target_path

    @staticmethod
    def download_url(
        url: str,
        retry_count: int = 3,
        timeout: float = 10,
        need_throttle: bool = False,
        rethrow: bool = True,
    ) -> requests.Response:
        for idx in range(retry_count):
            if need_throttle:
                hpyhrt_context.get_global_context().throttle.wait(url)
            try:
                mod_logger.debug(f"Try {idx} Begin: to download from: {url}")
                rsp = requests.get(url, timeout=timeout)
                mod_logger.debug(
                    f"Try {idx} End: finish download from: {url}, "
                    f"rsp.status_code {rsp.status_code}"
                )

                if not 200 <= rsp.status_code < 300:
                    mod_logger.warning(
                        f"Try {idx} failed: url {url}, rsp code {rsp.status_code}, text {rsp.text}"
                    )
                    continue

                return rsp
            except Exception as e:
                mod_logger.info(f"warn: download {url} failed: {e}")
                time.sleep(10)

                if idx == (retry_count - 1):
                    mod_logger.warning(f"error: download {url} failed")

                    if rethrow:
                        raise e

        raise Exception(f"Failed to download {url}")

    @staticmethod
    def find_abs_path(in_path: str, iter_lvl: int = 5) -> str | None:
        log_prefix = "find_abs_path:"

        mod_logger.info(f"{log_prefix} Enter to find {in_path}")

        abs_path: str | None = None

        if os.path.isabs(in_path):
            if os.path.exists(in_path):
                abs_path = in_path
        else:
            rel_path = in_path
            for _ in range(iter_lvl):
                if os.path.exists(rel_path):
                    abs_path = os.path.abspath(rel_path)
                    break
                rel_path = os.path.join("..", rel_path)

        if not abs_path:
            mod_logger.debug(f"{log_prefix} Leave without found")
            return None

        mod_logger.debug(f"{log_prefix} Leave with found {abs_path}")

        return abs_path

    @staticmethod
    def find_root_dir(
        config_file: str, check_dir_names: list[str] | None = None
    ) -> str:
        prev_dir = None
        cur_dir = os.path.dirname(config_file)

        if not check_dir_names:
            check_dir_names = ["data", "config"]

        while True:
            if cur_dir == prev_dir:
                raise Exception(f"Can't find root dir for {config_file}")

            all_check_pass = True

            for check_dir_name in check_dir_names:
                check_dir = os.path.join(cur_dir, check_dir_name)

                if not os.path.isdir(check_dir):
                    all_check_pass = False
                    break

            if all_check_pass:
                return cur_dir

            prev_dir = cur_dir
            cur_dir = os.path.abspath(os.path.join(cur_dir, ".."))

    @staticmethod
    def get_local_abs_file_path(file_path: str) -> str:
        # download from remote if not exist in local and cache in local
        if file_path.startswith("http"):
            local_abs_file_path = IOUtil.download_if_needed(
                file_path, hpyhrt_context.get_config_inst().data_cache_dir
            )
        else:
            local_abs_file_path = os.path.join(
                hpyhrt_context.get_config_inst().data_dir, file_path
            )
            if not os.path.exists(local_abs_file_path):
                raise Exception(
                    f"get_local_abs_file_path failed, file_path {local_abs_file_path} doesn't exist"
                )

        return local_abs_file_path

    @staticmethod
    def load_json_from_url(json_url: str, timeout: float = 10) -> Any:
        text = IOUtil.load_text_from_url(json_url, timeout=timeout)
        json_data = json.loads(text)
        return json_data

    @staticmethod
    def load_text_from_url(text_url: str, timeout: float = 10) -> str:
        if text_url.startswith("http"):
            return IOUtil.load_text_from_http_url(text_url, timeout)
        else:
            return IOUtil.load_text_from_file_url(text_url)

    @staticmethod
    def load_text_from_http_url(text_url: str, timeout: float = 10) -> str:
        headers: dict[str, str] = {}

        r = requests.get(text_url, headers=headers, timeout=timeout, verify=False)

        if 200 <= r.status_code < 300:
            text_content = r.text
            return text_content
        else:
            raise Exception(f"bad status code {r.status_code}")

    @staticmethod
    def load_text_from_file_url(text_url: str) -> str:
        with open(text_url, encoding="utf-8") as text_file:
            text_content = text_file.read()
            return text_content

    @staticmethod
    def maybe_make_dir(dirname: str) -> None:
        """Make a directory if it doesn't exist."""
        os.makedirs(dirname, exist_ok=True)

    @staticmethod
    def rm_file(file_path: str) -> None:
        log_prefix = "rm_file:"

        try:
            os.remove(file_path)
            mod_logger.debug(f"{log_prefix} file_path: {file_path}")
        except Exception as err:
            mod_logger.debug(f"{log_prefix} exception {err}")

    @staticmethod
    def rm_old(
        dir_path: str,
        days: float = 7,
        force: bool = False,
        filter_list: list[str] | None = None,
        remove_indeed: bool = True,
    ) -> None:
        log_prefix = "rm_old:"

        if not os.path.isdir(dir_path):
            return

        datetime_thres_obj = datetime.now() - timedelta(days=days)
        datetime_thres = datetime_thres_obj.timestamp()
        mod_logger.debug(
            f"{log_prefix} dir_path: {dir_path}, force_delete: {force}, "
            f"remove_indeed {remove_indeed}, datetime_thres {datetime_thres_obj}"
        )

        with os.scandir(dir_path) as it:
            for entry in it:
                stat_result = entry.stat()
                if False:
                    mod_logger.debug(
                        f"{log_prefix}, path: {entry.path}, st_mtime: {stat_result.st_mtime}, date_time_thres: {datetime_thres}, cur_time: {datetime.now()}"
                    )
                # 1604799194 is 2020-11-08 09:33:14
                if force or datetime_thres > stat_result.st_mtime > 1604799194:
                    if entry.is_dir():
                        mod_logger.debug(f"{log_prefix}, rm dir {entry.path}")
                        if remove_indeed:
                            if not filter_list or (
                                filter_list and entry.path not in filter_list
                            ):
                                shutil.rmtree(entry.path)
                                mod_logger.debug(
                                    f"{log_prefix}, rm dir {entry.path} done"
                                )
                    else:
                        mod_logger.debug(f"{log_prefix}, rm file {entry.path}")
                        if remove_indeed:
                            if not filter_list or (
                                filter_list and entry.path not in filter_list
                            ):
                                os.remove(entry.path)
                                mod_logger.debug(
                                    f"{log_prefix}, rm file {entry.path} done"
                                )

    @staticmethod
    def save_text_to_path(in_text: str, out_path: str) -> None:
        with open(out_path, "w", encoding="utf-8") as fh:
            fh.write(in_text)

    @staticmethod
    def unzip_to_dir(zip_path: str, dir_path: str | None = None) -> None:
        with zipfile.ZipFile(zip_path, "r") as zf:
            zf.extractall(dir_path)

    @staticmethod
    def zip_path(in_path: str, zip_file_path: str | None = None) -> None:
        def addToZip(zf: zipfile.ZipFile, path: str, zippath: str) -> None:
            if os.path.isfile(path):
                zf.write(path, zippath, zipfile.ZIP_DEFLATED)
            elif os.path.isdir(path):
                if zippath:
                    zf.write(path, zippath)
                for nm in sorted(os.listdir(path)):
                    addToZip(zf, os.path.join(path, nm), os.path.join(zippath, nm))

        if zip_file_path is None:
            zip_file_path = os.path.basename(in_path) + ".zip"
        with zipfile.ZipFile(zip_file_path, "w") as zf:
            zippath = os.path.basename(in_path)
            if not zippath:
                zippath = os.path.basename(os.path.dirname(in_path))
            if zippath in ("", os.curdir, os.pardir):
                zippath = ""
            addToZip(zf, in_path, zippath)



from typing import Any

from hpyhrtbase import hpyhrt_context


class ConfigUtil:
    @staticmethod
    def decode_invisible_config(orig_value: str, must_exist: bool = True) -> Any:
        """
        有些配置项含有敏感信息，为了打印log时候不打印它们，
        我们加了一层重定向，如果配置项有指定前缀，则真实
        配置存在重定向对应key下
        """
        config_inst = hpyhrt_context.get_config_inst()
        invisible_config_prefix = config_inst.invisible_config_prefix

        if orig_value.startswith(invisible_config_prefix):
            to_value = None

            prefix_len = len(invisible_config_prefix)

            config_key = orig_value[prefix_len:]

            if config_key != "" and hasattr(config_inst, config_key):
                to_value = config_inst[config_key]
            elif must_exist:
                raise Exception(f"{config_key} was not configured")

            return to_value
        else:
            return orig_value

    @staticmethod
    def override_config(config: Any, orig_value: str, key_prefix: str) -> tuple[str | None, Any]:
        """
        有些配置项含有敏感信息，为了打印log时候不打印它们，
        我们加了一层重定向，如果配置项有指定前缀，则真实
        配置存在重定向对应key下
        """
        if orig_value.startswith(key_prefix):
            to_value = None

            prefix_len = len(key_prefix)

            config_key = orig_value[prefix_len:]

            if config_key != "" and hasattr(config, config_key):
                to_value = config[config_key]

            return config_key, to_value
        else:
            return None, orig_value



import logging
import traceback
from logging.handlers import RotatingFileHandler
from typing import Any, NoReturn

from hpyhrtbase import hpyhrt_context


class LogUtil:
    @staticmethod
    def log_then_rethrown(log_prefix: str, e: Exception) -> NoReturn:
        prefix = "LogUtil"
        if str(e).startswith(prefix):
            raise e
        else:
            err_msg = f"{prefix} {log_prefix} failed: Exception {str(e)}"
            hpyhrt_context.get_robot_logger().error(err_msg)
            hpyhrt_context.get_robot_logger().info(traceback.format_exc())
            raise Exception(err_msg)

    @staticmethod
    def set_console_logger_level(lvl: Any) -> None:
        root_logger = logging.getLogger()
        for handler in root_logger.handlers:
            if not isinstance(handler, RotatingFileHandler):
                handler.setLevel(lvl)

    @staticmethod
    def set_logger_level(logger_name: str | None, level: Any) -> None:
        logging.getLogger(logger_name).setLevel(level)



import copy
import logging
import logging.config
import os
from logging.handlers import RotatingFileHandler
from typing import Any, cast

from hpyhrtbase import config
from hpyhrtbase.utils import IOUtil


def setup_logging(config_inst: config.Params) -> None:
    init_log_default_configs(config_inst)

    log_dir = config_inst.log_dir

    error_file_handler_on = True
    error_file_handler_idx = -1
    file_handler_on = True
    file_handler_idx = -1
    console_on = True

    log_config = copy.deepcopy(log_config_default)

    if "error_file_handler" in config_inst.log_handlers:
        handler_config = log_config["handlers"]["error_file_handler"]
        handler_config["filename"] = os.path.join(log_dir, "app_error.log")
        handler_config["backupCount"] = config_inst.log_backupCount
    else:
        del log_config["handlers"]["error_file_handler"]
        log_config["root"]["handlers"].remove("error_file_handler")
        error_file_handler_on = False

    if "file_handler" in config_inst.log_handlers:
        handler_config = log_config["handlers"]["file_handler"]
        handler_config["filename"] = os.path.join(log_dir, "app.log")
        handler_config["backupCount"] = config_inst.log_backupCount
    else:
        del log_config["handlers"]["file_handler"]
        log_config["root"]["handlers"].remove("file_handler")
        file_handler_on = False

    if "console" not in config_inst.log_handlers:
        del log_config["handlers"]["console"]
        log_config["root"]["handlers"].remove("console")
        console_on = False

    if not (error_file_handler_on or file_handler_on or console_on):
        raise Exception("at least enable one log handler")

    logging.config.dictConfig(log_config)
    root_logger = logging.getLogger()

    handler_idx = 0

    if console_on:
        handler_idx = 1

    if file_handler_on:
        file_handler_idx = handler_idx
        handler_idx += 1

    if error_file_handler_on:
        error_file_handler_idx = handler_idx

    if error_file_handler_idx >= 0:
        log_path = log_config["handlers"]["error_file_handler"]["filename"]
        if os.path.exists(log_path) and os.stat(log_path).st_size != 0:
            file_handler = cast(RotatingFileHandler, root_logger.handlers[error_file_handler_idx])
            file_handler.doRollover()

    if file_handler_idx >= 0:
        log_path = log_config["handlers"]["file_handler"]["filename"]
        if os.path.exists(log_path) and os.stat(log_path).st_size != 0:
            file_handler = cast(RotatingFileHandler, root_logger.handlers[file_handler_idx])
            file_handler.doRollover()

    if "logger_levels" in config_inst:
        for logger_level in config_inst.logger_levels:
            logging.getLogger(logger_level[0]).setLevel(logger_level[1])

    logging.debug("debug: App start logging")
    logging.info("info: App start logging")


def init_log_default_configs(config_inst: config.Params) -> None:
    if not hasattr(config_inst, "log_backupCount"):
        config_inst.log_backupCount = 20

    if not hasattr(config_inst, "log_dir"):
        config_inst.log_dir = os.path.join(config_inst.project_dir, "logs")

    if not hasattr(config_inst, "log_handlers"):
        config_inst.log_handlers = ["console", "file_handler", "error_file_handler"]

    if hasattr(config_inst, "log_dir"):
        IOUtil.maybe_make_dir(config_inst.log_dir)


log_config_default: dict[str, Any] = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "brief": {
            "class": "logging.Formatter",
            "format": "%(asctime)s.%(msecs)03d line%(lineno)s %(levelname)s %(message)s",
            "datefmt": "%Y-%m-%d %H:%M:%S",
        },
        "normal": {
            "class": "logging.Formatter",
            "format": "%(asctime)s.%(msecs)03d  %(levelname)-8s [%(name)s] %(funcName)s:%(lineno)d: %(message)s",
            "datefmt": "%Y-%m-%d %H:%M:%S",
        },
        "verbose": {
            "class": "logging.Formatter",
            "format": "%(asctime)s.%(msecs)03d  %(levelname)-8s [%(name)s] %(module)s:%(funcName)s:%(lineno)d: [%(process)d]: %(threadName)s: %(message)s",
            "datefmt": "%Y-%m-%d %H:%M:%S",
        },
    },
    "handlers": {
        "console": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "formatter": "normal",
            "stream": "ext://sys.stdout",
        },
        "file_handler": {
            "class": "logging.handlers.RotatingFileHandler",
            "level": "DEBUG",
            "formatter": "normal",
            "filename": "app.log",
            "maxBytes": 10485760,
            "backupCount": 20,
            "encoding": "utf8",
        },
        "error_file_handler": {
            "class": "logging.handlers.RotatingFileHandler",
            "level": "ERROR",
            "formatter": "normal",
            "filename": "app_error.log",
            "maxBytes": 10485760,
            "backupCount": 20,
            "encoding": "utf8",
        },
    },
    "loggers": {},
    "root": {
        "handlers": ["console", "file_handler", "error_file_handler"],
        "level": "INFO",
    },
}
